/*  
Theme Name: BuddyPress Default
Theme URI: http://buddypress.org/extend/themes/
Description: The default theme for BuddyPress.
Version: 1.1.1
Author: BuddyPress.org
Author URI: http://buddypress.org
Tags: buddypress, three-columns, white, orange, flexible-width
Template: bp-sn-parent
*/

/* Font styles */
@import url( ../bp-sn-parent/_inc/css/fonts/12px.css );

/* Layout styles for home page, internal pages and blog/directory pages */
@import url( ../bp-sn-parent/_inc/css/layout/home-page/3c-right.css );
@import url( ../bp-sn-parent/_inc/css/layout/internal-pages/3c-left.css );
@import url( ../bp-sn-parent/_inc/css/layout/blog-dir-pages/2c-right.css );

/* Default theme admin bar styles */
@import url( _inc/css/adminbar.css );

/* Default theme styles */
@import url( _inc/css/screen.css );

/* Uncomment the following line and add your custom styles to '_inc/css/custom.css' */
/* @import url( _inc/css/custom.css ); */